// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved.

namespace Microsoft.Samples.Commerce.Order.CommerceEntities
{
    using Microsoft.Commerce.Contracts;

    /// <summary>
    /// This class represents shipping methods configured for the commerce system (site).
    /// </summary>
    public class ShippingMethod : ICommerceEntity
    {
        /// <summary>
        /// defines the model name for this commerce entity
        /// </summary>
        public const string ModelNameDefinition = "ShippingMethod";

        /// <summary>
        /// The commerce entity.
        /// </summary>
        private CommerceEntity commerceEntity;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ShippingMethod"/> class.
        /// </summary>
        public ShippingMethod()
        {
            this.commerceEntity = new CommerceEntity(ShippingMethod.ModelNameDefinition);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ShippingMethod"/> class.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        public ShippingMethod(CommerceEntity commerceEntity)
        {
            this.commerceEntity = commerceEntity;
        }

        #endregion

        #region ICommerceEntity Members

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id of the entity.</value>
        public string Id
        {
            get { return this.commerceEntity.Id; }
            set { this.commerceEntity.Id = value; }
        }

        /// <summary>
        /// Gets or sets the type name of this object instance
        /// <remarks>Required so that the Builders can convert to the Multi-channel Commerce Foundation CommerceEntity base type</remarks>
        /// </summary>
        /// <value></value>
        public string ModelName
        {
            get { return this.commerceEntity.ModelName; }
            set { this.commerceEntity.ModelName = value; }
        }

        /// <summary>
        /// Gets or sets this objects collection of Properties and their corresponding values
        /// </summary>
        /// <value></value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly", Justification = "Collection setting is required")]
        public CommercePropertyCollection Properties
        {
            get { return this.commerceEntity.Properties; }
            set { this.commerceEntity.Properties = value; }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the shipping method description. This is an optional detailed description of the shipping method.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return this.commerceEntity.GetPropertyValue(PropertyName.Description) as string; }
            set { this.commerceEntity.SetPropertyValue(PropertyName.Description, value); }
        }

        /// <summary>
        /// Gets or sets the display name of the shipping method.
        /// </summary>
        /// <value>The display name.</value>
        /// <remarks>This is the name that is used in the UI to help to identify the shipping
        /// method for the user.</remarks>
        public string DisplayName
        {
            get { return this.commerceEntity.GetPropertyValue(PropertyName.DisplayName) as string; }
            set { this.commerceEntity.SetPropertyValue(PropertyName.DisplayName, value); }
        }

        #endregion

        #region Implicit operators
        /// <summary>
        /// Toes the user profile.
        /// </summary>
        /// <param name="commerceEntity">The commerce entity.</param>
        /// <returns></returns>
        public static implicit operator ShippingMethod(CommerceEntity commerceEntity)
        {
            System.Diagnostics.Debug.Assert(ShippingMethod.ModelNameDefinition.Equals(commerceEntity.ModelName), "Unexpected model name");
            return new ShippingMethod(commerceEntity);
        }
        #endregion

        /// <summary>
        /// Required so that the Builders can convert to the Multi-channel Commerce Foundation CommerceEntity base type using <see cref="ModelName"/>
        /// </summary>
        /// <returns>This instances base type</returns>
        public CommerceEntity ToCommerceEntity()
        {
            return this.commerceEntity;
        }

        #region Property key names

        /// <summary>
        /// Names of the properties exposed by this commerce entity.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1034:NestedTypesShouldNotBeVisible", Justification = "Needed for external access"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors", Justification = "Cannot make it private as it cannot be derived from")]
        public class PropertyName : CommerceEntity.PropertyName
        {
            #region Constants

            /// <summary>
            /// The name of the DisplayName property for a ShippingMethod.
            /// </summary>
            public const string DisplayName = "DisplayName";

            /// <summary>
            /// The name of the Description property for a ShippingMethod.
            /// </summary>
            public const string Description = "Description";

            #endregion
        }

        #endregion
    }
}
